import * as crypto from 'crypto';
import * as fs from 'fs';
import * as readline from 'readline';

// Checks to see if a public or private key exist.
if (
        !(fs.existsSync('privateKey.der')) ||
        !(fs.existsSync('publicKey.der'))
) {
        // Generates a private and public key pair for encryption.
        const {
                publicKey,
                privateKey
        } = crypto.generateKeyPairSync('rsa', {
                modulusLength: 2048,
        });

        // Writes the private key to a file.
        fs.writeFileSync('privateKey.der', privateKey.export({
                format: 'der',
                type: 'pkcs1'
        }));

        // Writes the public key to a file.
        fs.writeFileSync('publicKey.der', publicKey.export({
                format: 'der',
                type: 'pkcs1'
        }));
};

// Imports private key from a file.
const privateKey = crypto.createPrivateKey({
        format: 'der',
        key: fs.readFileSync('privateKey.der'),
        type: 'pkcs1'
});

// Imports public key from a file.
const publicKey = crypto.createPublicKey({
        format: 'der',
        key: fs.readFileSync('publicKey.der'),
        type: 'pkcs1'
});

// Lets the user know what their public key is.
console.log(`The following base64-encoded text is your public key, please use this on for any website that supports this method of multi-factor authentication:\n\n\x1b[33m${publicKey.export({
        format: 'der',
        key: publicKey.toString('base64'),
        type: 'pkcs1'
}).toString('base64')}\x1b[0m\n`);

// Initializes the system for accepting user input.
const rl = readline.createInterface({
        input: process.stdin,
        output: process.stdout
});

rl.setPrompt('');
rl.question('Please enter the base64-encoded challenge generated by the website you are logging into:\n\n', function (answer) {
        const decodedPIN = crypto.privateDecrypt({
                key: privateKey,
                padding: crypto.constants.RSA_PKCS1_PADDING
        }, Buffer.from(answer, 'base64'));

        console.log(`\x1b[0m\nMFA PIN #: \x1b[32m${decodedPIN}\x1b[0m`);
});
