import * as crypto from 'crypto';
import * as readline from 'readline';

// Initializes the system for accepting user input.
const rl = readline.createInterface({
        input: process.stdin,
        output: process.stdout
});

rl.setPrompt('');
rl.question('Please enter your public key to be used for multi-factor authentication:\n\n', function (answer) {
        let publicKey = null;
        try {
                publicKey = crypto.createPublicKey({
                        format: 'der',
                        key: Buffer.from(answer, 'base64'),
                        type: 'pkcs1'
                });

                const number = crypto.randomInt(0, 1000000).toString().padStart(6, 0);
                const challenge = crypto.publicEncrypt({
                        key: publicKey,
                        padding: crypto.constants.RSA_PKCS1_PADDING
                }, number).toString('base64');
                console.clear();
                console.log(`Base64-Encoded Challenge: \n\n\x1b[34m${challenge}\x1b[0m`);

                rl.question('\nPlease enter the answer to the challenge to login: ', function (answer) {
                        console.clear();
                        if (answer === number) {
                                console.log('Successfully logged in as Toshizuno!');
                        } else {
                                console.log('Login failed. Incorrect challenge answer.');
                        };
                });
        } catch {
                console.log('Imagine using an invalid public key :nerd:');
        };
});
